# ANTS-Hub - Copyright (C) Entr'ouvert

import datetime

import pytest
from django.core.management import call_command

from ants_hub.data.models import Config, Lieu, Plage, RendezVous, TypeDeRdv


@pytest.mark.parametrize(
    'endpoint', ['getManagedMeetingPoints', 'availableTimeSlots', 'searchApplicationIds']
)
def test_authentication(endpoint, django_app, db):
    django_app.get(f'/api/ants/{endpoint}', status=401)

    django_app.get(f'/api/ants/{endpoint}', headers={'X-Hub-Rdv-Auth-Token': 'abcd'}, status=401)
    Config.set(Config.REQUEST_FROM_ANTS_AUTH_TOKEN, 'abcd')
    django_app.get(f'/api/ants/{endpoint}', headers={'X-Hub-Rdv-Auth-Token': 'abcd'}, status=(200, 422))


class TestEndpoints:
    @pytest.fixture(autouse=True)
    def setup(self, db, settings, django_app, freezer):
        Config.set(Config.REQUEST_FROM_ANTS_AUTH_TOKEN, 'abcd')
        django_app.extra_environ['HTTP_X_HUB_RDV_AUTH_TOKEN'] = 'abcd'
        call_command('loaddata', 'fixtures/example1.json')
        freezer.move_to('2023-04-03T12:00:00+02:00')

    @pytest.fixture
    def lieu(self):
        return Lieu.objects.get()

    def test_managed_meeting_points(self, db, django_app):
        response = django_app.get('/api/ants/getManagedMeetingPoints')
        assert response.json == [
            {
                'city_name': 'Saint-Didier',
                'id': '1',
                'latitude': 2.0,
                'longitude': 1.0,
                'name': 'Mairie',
                'public_entry_address': '2 rue  du four',
                'zip_code': '99999',
                'website': 'https://saint-didier.fr/',
                'city_logo': 'https://saint-didier.fr/logo.png',
            },
        ]

    def test_available_time_slots(self, db, django_app, lieu):
        for i in range(5, 10):
            # open new RdV from 8th april to 12th april
            Plage.objects.create(
                lieu=lieu,
                date=datetime.date.today() + datetime.timedelta(days=i),
                horaires='10:00+02:00-12:00+02:00,14:00+02:00-17:00+02:00',
                duree=30,
                type_de_rdv=TypeDeRdv.CNI,
                personnes=1,
            )
        response = django_app.get(
            '/api/ants/availableTimeSlots',
            params={
                'meeting_point_ids': str(lieu.id),
                'start_date': '2023-04-09',
                'end_date': '2023-04-11',
            },
        )
        assert set(response.json.keys()) == {str(lieu.id)}
        slots = response.json[str(lieu.id)]
        # 4 meetings possible in the morning, 6 in the evening, for
        # 3 days = 30 slots
        assert len(slots) == 30
        for slot in slots:
            assert 9 <= datetime.datetime.fromisoformat(slot['datetime'].rstrip('Z')).day <= 11
            # remove the Z suffix
            assert slot['datetime'][:-1] in slot['callback_url']
            assert f'{lieu.slug}-{lieu.id}' in slot['callback_url']
            assert f'{lieu.collectivite.slug}-{lieu.collectivite.id}' in slot['callback_url']
        response = django_app.get(slots[-1]['callback_url'])
        assert response.location == 'https://saint-didier.fr/rdv/'
        lieu.rdv_url = 'https://saint-didier.fr/mairie/rdv/'
        lieu.save()
        response = django_app.get(slots[-1]['callback_url'])
        assert response.location == 'https://saint-didier.fr/mairie/rdv/'

    def test_available_time_slots_reason(self, db, django_app, lieu):
        Plage.objects.all().delete()
        kwargs = {
            'lieu': lieu,
            'date': datetime.date.today() + datetime.timedelta(days=1),
            'duree': 30,
        }
        Plage.objects.create(**kwargs, type_de_rdv=TypeDeRdv.CNI, horaires='10:00+02:00-10:30+02:00')
        Plage.objects.create(**kwargs, type_de_rdv=TypeDeRdv.PASSPORT, horaires='10:30+02:00-11:00+02:00')
        Plage.objects.create(**kwargs, type_de_rdv=TypeDeRdv.CNI_PASSPORT, horaires='11:00+02:00-11:30+02:00')

        # default is CNI
        response = django_app.get(
            '/api/ants/availableTimeSlots',
            params={
                'meeting_point_ids': str(lieu.id),
                'start_date': '2023-04-03',
                'end_date': '2023-04-05',
            },
        )
        assert len(response.json[str(lieu.id)]) == 1
        assert {x['datetime'] for x in response.json[str(lieu.id)]} == {'2023-04-04T10:00:00Z'}

        # CNI matches CNI and CNI-PASSPORT
        response = django_app.get(
            '/api/ants/availableTimeSlots',
            params={
                'meeting_point_ids': str(lieu.id),
                'start_date': '2023-04-03',
                'end_date': '2023-04-05',
                'reason': 'CNI',
            },
        )
        assert len(response.json[str(lieu.id)]) == 1
        assert {x['datetime'] for x in response.json[str(lieu.id)]} == {'2023-04-04T10:00:00Z'}

        response = django_app.get(
            '/api/ants/availableTimeSlots',
            params={
                'meeting_point_ids': str(lieu.id),
                'start_date': '2023-04-03',
                'end_date': '2023-04-05',
                'reason': 'PASSPORT',
            },
        )
        assert len(response.json[str(lieu.id)]) == 1
        assert {x['datetime'] for x in response.json[str(lieu.id)]} == {'2023-04-04T10:30:00Z'}

        response = django_app.get(
            '/api/ants/availableTimeSlots',
            params={
                'meeting_point_ids': str(lieu.id),
                'start_date': '2023-04-03',
                'end_date': '2023-04-05',
                'reason': 'CNI-PASSPORT',
            },
        )
        assert len(response.json[str(lieu.id)]) == 1
        assert {x['datetime'] for x in response.json[str(lieu.id)]} == {'2023-04-04T11:00:00Z'}

    def test_available_time_slots_documents_number(self, db, django_app, lieu):
        Plage.objects.all().delete()
        kwargs = {
            'lieu': lieu,
            'date': datetime.date.today() + datetime.timedelta(days=1),
            'duree': 30,
            'type_de_rdv': TypeDeRdv.CNI,
        }
        Plage.objects.create(**kwargs, personnes=1, horaires='10:00+02:00-10:30+02:00')
        Plage.objects.create(**kwargs, personnes=2, horaires='10:30+02:00-11:00+02:00')
        Plage.objects.create(**kwargs, personnes=3, horaires='11:00+02:00-11:30+02:00')
        Plage.objects.create(**kwargs, personnes=4, horaires='12:00+02:00-12:30+02:00')
        Plage.objects.create(**kwargs, personnes=5, horaires='12:30+02:00-13:00+02:00')

        # default is 1 person
        response = django_app.get(
            '/api/ants/availableTimeSlots',
            params={
                'meeting_point_ids': str(lieu.id),
                'start_date': '2023-04-03',
                'end_date': '2023-04-05',
            },
        )
        assert len(response.json[str(lieu.id)]) == 1
        assert {x['datetime'] for x in response.json[str(lieu.id)]} == {'2023-04-04T10:00:00Z'}

        response = django_app.get(
            '/api/ants/availableTimeSlots',
            params={
                'meeting_point_ids': str(lieu.id),
                'start_date': '2023-04-03',
                'end_date': '2023-04-05',
                'documents_number': 1,
            },
        )
        assert len(response.json[str(lieu.id)]) == 1
        assert {x['datetime'] for x in response.json[str(lieu.id)]} == {'2023-04-04T10:00:00Z'}

        response = django_app.get(
            '/api/ants/availableTimeSlots',
            params={
                'meeting_point_ids': str(lieu.id),
                'start_date': '2023-04-03',
                'end_date': '2023-04-05',
                'documents_number': 2,
            },
        )
        assert len(response.json[str(lieu.id)]) == 1
        assert {x['datetime'] for x in response.json[str(lieu.id)]} == {'2023-04-04T10:30:00Z'}

        response = django_app.get(
            '/api/ants/availableTimeSlots',
            params={
                'meeting_point_ids': str(lieu.id),
                'start_date': '2023-04-03',
                'end_date': '2023-04-05',
                'documents_number': 3,
            },
        )
        assert len(response.json[str(lieu.id)]) == 1
        assert {x['datetime'] for x in response.json[str(lieu.id)]} == {'2023-04-04T11:00:00Z'}

        # implicit maximum is 5
        response = django_app.get(
            '/api/ants/availableTimeSlots',
            params={
                'meeting_point_ids': str(lieu.id),
                'start_date': '2023-04-03',
                'end_date': '2023-04-05',
                'documents_number': 6,
            },
        )
        assert len(response.json[str(lieu.id)]) == 1
        assert {x['datetime'] for x in response.json[str(lieu.id)]} == {'2023-04-04T12:30:00Z'}

    def test_search_application_ids(self, db, django_app, lieu):
        RendezVous.objects.create(
            uuid="7621a90a-2dd3-44e5-9df7-879abddeaad5",
            lieu=lieu,
            identifiant_predemande='123456',
            date=datetime.datetime.fromisoformat('2023-04-11T11:00:00+02:00'),
            gestion_url='https://monrdv.fr/gestion/',
            annulation_url='https://monrdv.fr/annulation/',
        )
        response = django_app.get(
            '/api/ants/searchApplicationIds',
            params={
                'application_ids': ['abcd', '123456'],
            },
        )
        assert response.json == {
            '123456': [
                {
                    'cancel_url': 'http://testserver/rdv/saint-didier-1/mairie-1/2023-04-11T11:00:00+02:00/'
                    'annulation/7621a90a-2dd3-44e5-9df7-879abddeaad5/',
                    'datetime': '2023-04-11T11:00:00Z',
                    'management_url': 'http://testserver/rdv/saint-didier-1/mairie-1/2023-04-11T11:00:00+02:00/'
                    'gestion/7621a90a-2dd3-44e5-9df7-879abddeaad5/',
                    'meeting_point': 'Mairie',
                }
            ],
            'abcd': [
                {
                    'cancel_url': 'http://testserver/rdv/saint-didier-1/mairie-1/2023-04-03T12:15:00+02:00/'
                    'annulation/7cace277-9157-4fbc-9705-45522984805d/',
                    'datetime': '2023-04-03T12:15:00Z',
                    'management_url': 'http://testserver/rdv/saint-didier-1/mairie-1/2023-04-03T12:15:00+02:00/'
                    'gestion/7cace277-9157-4fbc-9705-45522984805d/',
                    'meeting_point': 'Mairie',
                }
            ],
        }
        locations = {
            identifiant_predemande: [
                {k: django_app.get(v).location for k, v in rdv.items() if k.endswith('_url')}
                for rdv in response.json[identifiant_predemande]
            ]
            for identifiant_predemande in response.json
        }
        assert locations == {
            '123456': [
                {
                    'cancel_url': 'https://monrdv.fr/annulation/',
                    'management_url': 'https://monrdv.fr/gestion/',
                }
            ],
            'abcd': [
                {
                    'cancel_url': 'https://saint-didier.fr/mairie/annulation/',
                    'management_url': 'https://saint-didier/gestion/',
                }
            ],
        }

    class TestParameterError:
        def test_available_time_slots(self, db, django_app):
            django_app.get('/api/ants/availableTimeSlots', params={}, status=422)
            django_app.get('/api/ants/availableTimeSlots', params={'meeting_point_ids': 'xyz'}, status=422)
            django_app.get('/api/ants/availableTimeSlots', params={'meeting_point_ids': '1'}, status=422)
            django_app.get(
                '/api/ants/availableTimeSlots',
                params={'meeting_point_ids': '1', 'start_date': 'abcd'},
                status=422,
            )
            django_app.get(
                '/api/ants/availableTimeSlots',
                params={'meeting_point_ids': '1', 'start_date': '2023-03-05'},
                status=422,
            )
            django_app.get(
                '/api/ants/availableTimeSlots',
                params={'meeting_point_ids': '1', 'start_date': '2023-04-05'},
                status=422,
            )
            django_app.get(
                '/api/ants/availableTimeSlots',
                params={'meeting_point_ids': '1', 'start_date': '2023-04-05', 'end_date': 'abcd'},
                status=200,
            )
            django_app.get(
                '/api/ants/availableTimeSlots',
                params={'meeting_point_ids': '1', 'start_date': 'abcd', 'end_date': 'abcd'},
                status=200,
            )
            django_app.get(
                '/api/ants/availableTimeSlots',
                params={'meeting_point_ids': '1', 'start_date': '2023-04-05', 'end_date': '2023-02-01'},
                status=200,
            )
            django_app.get(
                '/api/ants/availableTimeSlots',
                params={
                    'meeting_point_ids': '1',
                    'start_date': '2023-04-05',
                    'end_date': '2023-04-10',
                    'reason': 'xyz',
                },
                status=200,
            )
            django_app.get(
                '/api/ants/availableTimeSlots',
                params={
                    'meeting_point_ids': '1',
                    'start_date': '2023-04-05',
                    'end_date': '2023-04-10',
                    'reason': 'xyz',
                    'documents_number': 'zbc',
                },
                status=200,
            )
            django_app.get(
                '/api/ants/availableTimeSlots',
                params={
                    'meeting_point_ids': '1',
                    'start_date': '2023-04-05',
                    'end_date': '2023-04-10',
                    'reason': 'xyz',
                    'documents_number': '10',
                },
                status=200,
            )

        def test_search_application_ids(self, db, django_app):
            django_app.get('/api/ants/searchApplicationIds', params={}, status=422)
